@@ -28,6 +28,7 @@ from rdkit.Chem import AllChem, rdFMCS
 from warnings import warn
 import re, string
 from typing import Optional
+from rdkit_to_params.entries import CONNECTEntry
 
 from ._rdkit_rename import _RDKitRenameMixin
 
@@ -58,6 +59,7 @@ class _RDKitPrepMixin(_RDKitRenameMixin):
         self.TYPE.append('LIGAND')
         if name is not None:
             self.NAME = name
+            self.mol.SetProp('_Name', name)
         self.fix_mol()
         # conversion elsewhere
         return self
@@ -74,6 +76,8 @@ class _RDKitPrepMixin(_RDKitRenameMixin):
         """
         cls.log.debug('`load_smiles` called...')
         mol = Chem.MolFromSmiles(smiles)
+        if mol is None:
+            raise ValueError('The SMILES string could not be converted')
         mol.SetProp('_Name', name)
         mol = Chem.AddHs(mol)
         # cannot embed more than one dummy
@@ -233,11 +237,11 @@ class _RDKitPrepMixin(_RDKitRenameMixin):
                 elif atom.GetIsAromatic():  # could also be NtrR...
                     atom.SetProp('_rType', 'Ntrp')
                 elif atom.GetHybridization() == Chem.HybridizationType.SP3 and len(Hs) < 3:
-                    atom.SetProp('_rType', 'Nbb')
+                    atom.SetProp('_rType', 'Npro')  # Nbb is a SP2
                 elif atom.GetHybridization() == Chem.HybridizationType.SP3:
                     atom.SetProp('_rType', 'Nlys')
                 elif atom.GetHybridization() == Chem.HybridizationType.SP2:
-                    atom.SetProp('_rType', 'Npro')  # or Narg if the orbitals are funky...
+                    atom.SetProp('_rType', 'NH2O')  # or Narg if the orbitals are funky...
                 else:
                     raise ValueError(f'No idea what this nitrogen {atom.GetHybridization()} is')
             elif symbol == 'O':
@@ -434,6 +438,13 @@ class _RDKitPrepMixin(_RDKitRenameMixin):
             self.retype_by_name(aa_map)
             # change conn
             elemental['CONN'] = 2  # when LOWER and UPPER exist the CONN is CONN3.
+            # add properties
+            self.PROPERTIES.append('PROTEIN')
+            self.PROPERTIES.append('ALPHA_AA')
+            self.PROPERTIES.append('L_AA')
+            # HYDROPHOBIC ALIPHATIC
+            self.FIRST_SIDECHAIN_ATOM.append('CB')
+            self.BACKBONE_AA.append('ALA')
 
     def _greekify(self):
         greek = list('ABGDEZHTIKLMNXOPRS')
@@ -459,7 +470,7 @@ class _RDKitPrepMixin(_RDKitRenameMixin):
                     name = f'{atom.GetSymbol(): >2}{greek[k]}{l[i]}'
                     self.rename_atom(greekdex[k][0], name)
             else:
-                pass # what????
+                pass # no renaming. This is insane corner case. A 36 HA AA is madness.
 
 
     def _fix_atom_names(self):
@@ -625,3 +636,5 @@ class _RDKitPrepMixin(_RDKitRenameMixin):
             if i in changed:
                 atom.SetAtomicNum(0)
         self.fix_mol()
+
+
